﻿namespace TaskPlanner.BusinessLogic.Test.Mocks
{
	using System;
	using System.Collections.ObjectModel;
	using System.Linq;
	using System.Linq.Expressions;
	using TaskPlanner.Entities;
	using TaskPlanner.Repository.Interfaces;

	/// <summary>
	/// Represents a mock for task repository.
	/// </summary>
	internal class TaskRepository : ITaskRepository
	{
		#region Fields
		private bool _isGetAllTasksCalled;
		private bool _isSaveTaskCalled;
		private bool _isGetTasksTreeCalled;
		private bool _isGetActiveTasksCalled;
		private Collection<Task> _tasks;
		private bool _isDeleteTaskCalled;
		#endregion

		#region Constructors
		/// <summary>
		/// Initializes a new instance of the <see cref="TaskRepository"/> class.
		/// </summary>
		internal TaskRepository()
		{
			_tasks = new Collection<Task>();
		}
		#endregion

		#region Properties
		/// <summary>
		/// Gets a value indicating whether method GetAllTasks was called.
		/// </summary>
		/// <value>
		/// 	<c>True</c> if method GetAllTasks was called; otherwise, <c>false</c>.
		/// </value>
		internal bool IsGetAllTasksCalled
		{
			get
			{
				return _isGetAllTasksCalled;
			}
		}

		/// <summary>
		/// Gets a value indicating whether method SaveTask was called.
		/// </summary>
		/// <value>
		/// 	<c>True</c> if method SaveTask was called; otherwise, <c>false</c>.
		/// </value>
		internal bool IsSaveTaskCalled
		{
			get
			{
				return _isSaveTaskCalled;
			}
		}

		/// <summary>
		/// Gets a value indicating whether method GetTasksTree was called.
		/// </summary>
		/// <value>
		/// 	<c>True</c> if method GetTasksTree was called; otherwise, <c>false</c>.
		/// </value>
		internal bool IsGetTasksTreeCalled
		{
			get
			{
				return _isGetTasksTreeCalled;
			}
		}

		/// <summary>
		/// Gets a value indicating whether method GetActiveTasks was called.
		/// </summary>
		/// <value>
		/// 	<c>True</c> if method GetActiveTasks was called; otherwise, <c>false</c>.
		/// </value>
		internal bool IsGetActiveTasksCalled
		{
			get
			{
				return _isGetActiveTasksCalled;
			}
		}

		/// <summary>
		/// Gets a value indicating whether method DeleteTask was called.
		/// </summary>
		/// <value>
		/// 	<c>True</c> if method DeleteTask was called; otherwise, <c>false</c>.
		/// </value>
		internal bool IsDeleteTaskCalled
		{
			get
			{
				return _isDeleteTaskCalled;
			}
		}
		#endregion

		#region Methods
		/// <summary>
		/// Gets all tasks.
		/// </summary>
		/// <returns>The all tasks in repository.</returns>
		public Collection<Task> GetAllTasks()
		{
			_isGetAllTasksCalled = true;
			return _tasks;
		}

		/// <summary>
		/// Saves the task.
		/// </summary>
		/// <param name="task">The task to be saved.</param>
		public void SaveTask(Task task)
		{
			_isSaveTaskCalled = true;
			_tasks.Add(task);
		}

		/// <summary>
		/// Gets the topmost level tasks and assigns their children recursively.
		/// </summary>
		/// <returns>Tasks tree.</returns>
		public Collection<Task> GetTasksTree()
		{
			_isGetTasksTreeCalled = true;
			return new Collection<Task>(
				(from item in _tasks
				 where item.Parent == null
				 select item).ToList());
		}

		/// <summary>
		/// Gets the active tasks.
		/// </summary>
		/// <typeparam name="TOrder">The type of the order.</typeparam>
		/// <param name="filter">The filter.</param>
		/// <param name="order">The order.</param>
		/// <returns>Active tasks.</returns>
		public Collection<Task> GetActiveTasks<TOrder>(
			Expression<Func<Task, bool>> filter,
			Expression<Func<Task, TOrder>> order)
		{
			return new Collection<Task>(
				(from item in _tasks.AsQueryable<Task>().Where(filter).OrderBy(order)
				 where item.CompletedPercentage < 100
				 select item).ToList());
		}

		/// <summary>
		/// Gets the active tasks.
		/// </summary>
		/// <typeparam name="TOrder">The type of the order.</typeparam>
		/// <param name="order">The order.</param>
		/// <returns>Active tasks in specified order.</returns>
		public Collection<Task> GetActiveTasks<TOrder>(Expression<Func<Task, TOrder>> order)
		{
			_isGetActiveTasksCalled = true;
			return new Collection<Task>(
				(from item in _tasks.AsQueryable<Task>().OrderBy(order)
				 where item.CompletedPercentage < 100
				 select item).ToList());
		}

		/// <summary>
		/// Deletes the task.
		/// </summary>
		/// <param name="task">The task to be deleted.</param>
		public void DeleteTask(Task task)
		{
			_tasks.Remove(task);
			_isDeleteTaskCalled = true;
		}
		#endregion
	}
}
