﻿namespace TaskPlanner.BusinessLogic.Test
{
	using System;
	using System.Collections.Generic;
	using System.Collections.ObjectModel;
	using System.Threading;
	using Microsoft.VisualStudio.TestTools.UnitTesting;
	using TaskPlanner.BusinessLogic;
	using TaskPlanner.BusinessLogic.Interfaces;
	using TaskPlanner.BusinessLogic.Test.Mocks;
	using TaskPlanner.Entities;
	using TaskPlanner.Repository.Interfaces;

	/// <summary>
	/// This is a test class for TaskManagerTest and is intended
	/// to contain all TaskManagerTest Unit Tests.
	/// </summary>
	[TestClass()]
	public class TaskManagerTest
	{
		#region Fields
		private TestContext _testContextInstance;
		#endregion

		#region Properties
		/// <summary>
		/// Gets or sets the test context which provides
		/// information about and functionality for the current test run.
		/// </summary>
		/// <value>The test context.</value>
		public TestContext TestContext
		{
			get
			{
				return _testContextInstance;
			}

			set
			{
				_testContextInstance = value;
			}
		}
		#endregion

		#region Methods
		/// <summary>
		/// A test for TaskManager Constructor.
		/// </summary>
		[TestMethod()]
		public void TaskManagerConstructorTest()
		{
			ITaskRepository taskRepository = null;
			IUsualAvailabilityRepository usualAvailabilityRepository = null;
			ISpecificAvailabilityRepository specificAvailabilityRepository = null;
			Utils.AssertException(() => new TaskManager(taskRepository, usualAvailabilityRepository, specificAvailabilityRepository), typeof(ArgumentNullException));
		}

		/// <summary>
		/// A test for CreateNewTask.
		/// </summary>
		[TestMethod()]
		public void CreateNewTaskTest()
		{
			TaskManager target = new TaskManager(new TaskRepository(), new UsualAvailabilityRepository(), new SpecificAvailabilityRepository());
			Task actual = target.CreateNewTask();
			Guid notExpectedTaskId = Guid.Empty;
			Assert.AreNotEqual(notExpectedTaskId, actual.Id);
			DateTime expectedDate = DateTime.Today;
			Assert.AreEqual(expectedDate, actual.StartDate);
			Assert.AreEqual(expectedDate, actual.DueDate);
			Assert.AreEqual(expectedDate, actual.EndDate);
		}

		/// <summary>
		/// A test for GetAllTasks.
		/// </summary>
		[TestMethod()]
		public void GetAllTasksTest()
		{
			TaskRepository taskRepository = new TaskRepository();
			UsualAvailabilityRepository usualAvailabilityRepository = new UsualAvailabilityRepository();
			SpecificAvailabilityRepository specificAvailabilityRepository = new SpecificAvailabilityRepository();
			TaskManager target = new TaskManager(taskRepository, usualAvailabilityRepository, specificAvailabilityRepository);
			target.GetAllTasks();
			Assert.IsTrue(taskRepository.IsGetAllTasksCalled);
		}

		/// <summary>
		/// A test for GetTasksTree.
		/// </summary>
		[TestMethod()]
		public void GetTasksTreeTest()
		{
			TaskRepository taskRepository = new TaskRepository();
			UsualAvailabilityRepository usualAvailabilityRepository = new UsualAvailabilityRepository();
			SpecificAvailabilityRepository specificAvailabilityRepository = new SpecificAvailabilityRepository();
			TaskManager target = new TaskManager(taskRepository, usualAvailabilityRepository, specificAvailabilityRepository);
			target.GetTasksTree();
			Assert.IsTrue(taskRepository.IsGetTasksTreeCalled);
		}

		/// <summary>
		/// A test for SaveTask.
		/// </summary>
		[TestMethod()]
		public void SaveTaskTest()
		{
			TaskRepository taskRepository = new TaskRepository();
			UsualAvailabilityRepository usualAvailabilityRepository = new UsualAvailabilityRepository();
			SpecificAvailabilityRepository specificAvailabilityRepository = new SpecificAvailabilityRepository();
			TaskManager target = new TaskManager(taskRepository, usualAvailabilityRepository, specificAvailabilityRepository);
			Task task = null;
			Utils.AssertException(() => target.SaveTask(task), typeof(ArgumentNullException));
			task = new Task();
			target.SaveTask(task);
			Assert.IsTrue(taskRepository.IsSaveTaskCalled);
		}

		/// <summary>
		/// A test for DeleteTask.
		/// </summary>
		[TestMethod()]
		public void DeleteTaskTest()
		{
			TaskRepository taskRepository = new TaskRepository();
			UsualAvailabilityRepository usualAvailabilityRepository = new UsualAvailabilityRepository();
			SpecificAvailabilityRepository specificAvailabilityRepository = new SpecificAvailabilityRepository();
			TaskManager target = new TaskManager(taskRepository, usualAvailabilityRepository, specificAvailabilityRepository);
			Task task = null;
			Utils.AssertException(() => target.DeleteTask(task), typeof(ArgumentNullException));
			task = new Task();
			target.SaveTask(task);
			target.DeleteTask(task);
			Assert.IsTrue(taskRepository.IsDeleteTaskCalled);
		}

		/// <summary>
		/// A test for ApplyEstimatedTimeChange.
		/// </summary>
		[TestMethod()]
		public void ApplyEstimatedTimeChangeTest()
		{
			ITaskRepository taskRepository = new TaskRepository();
			IUsualAvailabilityRepository usualAvailabilityRepository = new UsualAvailabilityRepository();
			ISpecificAvailabilityRepository specificAvailabilityRepository = new SpecificAvailabilityRepository();
			TaskManager target = new TaskManager(taskRepository, usualAvailabilityRepository, specificAvailabilityRepository);
			Utils.AssertException(() => target.ApplyEstimatedTimeChange(null, false), typeof(ArgumentNullException));
			DateTime availabilityDate = DateTime.Today;
			SpecificAvailability availability = new SpecificAvailability();
			availability.Date = availabilityDate;
			availability.Availability = 2;
			specificAvailabilityRepository.AddAvailability(availability);
			availability = new SpecificAvailability();
			availabilityDate = availabilityDate.AddDays(1);
			availability.Date = availabilityDate;
			availability.Availability = 6;
			specificAvailabilityRepository.AddAvailability(availability);
			availability = new SpecificAvailability();
			availabilityDate = availabilityDate.AddDays(1);
			availability.Date = availabilityDate;
			availability.Availability = 2;
			specificAvailabilityRepository.AddAvailability(availability);
			availability = new SpecificAvailability();
			availabilityDate = availabilityDate.AddDays(1);
			availability.Date = availabilityDate;
			availability.Availability = 6;
			specificAvailabilityRepository.AddAvailability(availability);
			availability = new SpecificAvailability();
			availabilityDate = availabilityDate.AddDays(1);
			availability.Date = availabilityDate;
			availability.Availability = 2;
			specificAvailabilityRepository.AddAvailability(availability);
			availability = new SpecificAvailability();
			availabilityDate = availabilityDate.AddDays(1);
			availability.Date = availabilityDate;
			availability.Availability = 6;
			specificAvailabilityRepository.AddAvailability(availability);
			availability = new SpecificAvailability();
			availabilityDate = availabilityDate.AddDays(1);
			availability.Date = availabilityDate;
			availability.Availability = 2;
			specificAvailabilityRepository.AddAvailability(availability);
			availability = new SpecificAvailability();
			availabilityDate = availabilityDate.AddDays(1);
			availability.Date = availabilityDate;
			availability.Availability = 6;
			specificAvailabilityRepository.AddAvailability(availability);
			availability = new SpecificAvailability();
			availabilityDate = availabilityDate.AddDays(1);
			availability.Date = availabilityDate;
			availability.Availability = 2;
			specificAvailabilityRepository.AddAvailability(availability);
			availability = new SpecificAvailability();
			availabilityDate = availabilityDate.AddDays(1);
			availability.Date = availabilityDate;
			availability.Availability = 6;
			specificAvailabilityRepository.AddAvailability(availability);
			Task task = new Task();
			task.Id = Guid.NewGuid();
			task.StartDate = DateTime.Today;
			task.DueDate = DateTime.Today.AddDays(2);
			task.EstimatedTime = 5;
			taskRepository.SaveTask(task);
			task = new Task();
			task.Id = Guid.NewGuid();
			task.StartDate = DateTime.Today;
			task.DueDate = DateTime.Today.AddDays(2);
			task.EstimatedTime = 2.5F;
			taskRepository.SaveTask(task);
			task = new Task();
			task.Id = Guid.NewGuid();
			task.StartDate = DateTime.Today.AddDays(4);
			task.DueDate = DateTime.Today.AddDays(5);
			task.EstimatedTime = 4;
			taskRepository.SaveTask(task);
			task = new Task();
			task.Id = Guid.NewGuid();
			task.StartDate = DateTime.Today;
			task.EstimatedTime = 12.5F;
			ManualResetEvent eventWaiter = new ManualResetEvent(false);
			target.TaskCorrectionsRequested += (sender, e) => eventWaiter.Set();
			TaskCorrectionResult validationResult = target.ApplyEstimatedTimeChange(task, false);
			Assert.IsNotNull(validationResult);
			Assert.IsTrue(validationResult.IsCorrectionMade);
			Assert.IsTrue(eventWaiter.WaitOne(5000));
			Assert.AreEqual(TaskCorrectionReason.DueDateSuggestion, validationResult.CorrectionReason);
			Assert.IsNotNull(validationResult.CorrectedTask);
			Assert.AreEqual(DateTime.Today.AddDays(9), validationResult.CorrectedTask.DueDate);
			task.DueDate = DateTime.Today.AddDays(4);
			validationResult = target.ApplyEstimatedTimeChange(task, true);
			Assert.IsNotNull(validationResult);
			Assert.IsTrue(validationResult.IsCorrectionMade);
			Assert.AreEqual(TaskCorrectionReason.DueDateIsTooSmall, validationResult.CorrectionReason);
			Assert.IsNotNull(validationResult.CorrectedTask);
			Assert.AreEqual(DateTime.Today.AddDays(5), validationResult.CorrectedTask.DueDate);
			task.DueDate = DateTime.Today.AddDays(6);
			validationResult = target.ApplyEstimatedTimeChange(task, true);
			Assert.IsNotNull(validationResult);
			Assert.IsFalse(validationResult.IsCorrectionMade);
		}

		/// <summary>
		/// A test for ApplyCompletedPercentageChange.
		/// </summary>
		[TestMethod()]
		public void ApplyCompletedPercentageChangeTest()
		{
			TaskRepository taskRepository = new TaskRepository();
			UsualAvailabilityRepository usualAvailabilityRepository = new UsualAvailabilityRepository();
			SpecificAvailabilityRepository specificAvailabilityRepository = new SpecificAvailabilityRepository();
			TaskManager target = new TaskManager(taskRepository, usualAvailabilityRepository, specificAvailabilityRepository);
			Utils.AssertException(() => target.ApplyCompletedPercentageChange(null), typeof(ArgumentNullException));
			Task task = new Task();
			task.CompletedPercentage = 50;
			task.CompletedTime = 2;
			task.EstimatedTime = 10;
			ManualResetEvent eventWaiter = new ManualResetEvent(false);
			target.TaskCorrectionsRequested += (sender, e) => eventWaiter.Set();
			TaskCorrectionResult correctionResult = target.ApplyCompletedPercentageChange(task);
			Assert.IsNotNull(correctionResult);
			Assert.IsTrue(correctionResult.IsCorrectionMade);
			Assert.IsTrue(eventWaiter.WaitOne(5000));
			Assert.AreEqual(TaskCorrectionReason.EstimatedTimeSuggestion, correctionResult.CorrectionReason);
			Assert.IsNotNull(correctionResult.CorrectedTask);
			Assert.AreEqual(4, correctionResult.CorrectedTask.EstimatedTime);
			task.EstimatedTime = 4;
			correctionResult = target.ApplyCompletedPercentageChange(task);
			Assert.IsNotNull(correctionResult);
			Assert.IsFalse(correctionResult.IsCorrectionMade);
			task.CompletedPercentage = 100;
			correctionResult = target.ApplyCompletedPercentageChange(task);
			Assert.IsNotNull(correctionResult);
			Assert.IsTrue(correctionResult.IsCorrectionMade);
			Assert.AreEqual(TaskCorrectionReason.EndDateSuggestion, correctionResult.CorrectionReason);
			Assert.IsNotNull(correctionResult.CorrectedTask);
			Assert.AreEqual(DateTime.Today, correctionResult.CorrectedTask.EndDate);
		}

		/// <summary>
		/// A test for ApplyCompletedTimeChange.
		/// </summary>
		[TestMethod()]
		public void ApplyCompletedTimeChangeTest()
		{
			TaskRepository taskRepository = new TaskRepository();
			UsualAvailabilityRepository usualAvailabilityRepository = new UsualAvailabilityRepository();
			SpecificAvailabilityRepository specificAvailabilityRepository = new SpecificAvailabilityRepository();
			TaskManager target = new TaskManager(taskRepository, usualAvailabilityRepository, specificAvailabilityRepository);
			Utils.AssertException(() => target.ApplyCompletedTimeChange(null), typeof(ArgumentNullException));
			Task task = new Task();
			task.CompletedPercentage = 30;
			task.CompletedTime = 5;
			task.EstimatedTime = 10;
			ManualResetEvent eventWaiter = new ManualResetEvent(false);
			target.TaskCorrectionsRequested += (sender, e) => eventWaiter.Set();
			TaskCorrectionResult correctionResult = target.ApplyCompletedTimeChange(task);
			Assert.IsNotNull(correctionResult);
			Assert.IsTrue(correctionResult.IsCorrectionMade);
			Assert.IsTrue(eventWaiter.WaitOne(5000));
			Assert.AreEqual(TaskCorrectionReason.CompletedPercentageSuggestion, correctionResult.CorrectionReason);
			Assert.IsNotNull(correctionResult.CorrectedTask);
			Assert.AreEqual(50, correctionResult.CorrectedTask.CompletedPercentage);
			task.CompletedPercentage = 50;
			correctionResult = target.ApplyCompletedTimeChange(task);
			Assert.IsNotNull(correctionResult);
			Assert.IsFalse(correctionResult.IsCorrectionMade);
		}

		/// <summary>
		/// A test for CalculateRemainingAvailability.
		/// </summary>
		[TestMethod()]
		[DeploymentItem("TaskPlanner.BusinessLogic.dll")]
		public void CalculateRemainingAvailabilityTest()
		{
			Collection<Task> tasks = new Collection<Task>();
			Task task = new Task();
			task.CompletedTime = 1;
			task.EstimatedTime = 5;
			task.DueDate = DateTime.Today.AddDays(1);
			tasks.Add(task);
			Collection<UsualAvailability> usualAvailabilities = new Collection<UsualAvailability>();
			UsualAvailability usualAvailability = new UsualAvailability();
			usualAvailability.Availability = 4;
			usualAvailability.DayOfWeek = DateTime.Today.DayOfWeek.ToString();
			usualAvailabilities.Add(usualAvailability);
			Collection<SpecificAvailability> specificAvailabilities = new Collection<SpecificAvailability>();
			SpecificAvailability specificAvailability = new SpecificAvailability();
			specificAvailability.Availability = 4;
			specificAvailability.Date = DateTime.Today.AddDays(1);
			specificAvailabilities.Add(specificAvailability);
			Dictionary<DateTime, float> actual = TaskManager_Accessor.CalculateRemainingAvailability(
				tasks,
				usualAvailabilities,
				specificAvailabilities);
			Assert.IsNotNull(actual);
			Assert.AreEqual(2, actual.Count);
			Assert.AreEqual(2, actual[DateTime.Today]);
			Assert.AreEqual(2, actual[DateTime.Today.AddDays(1)]);
		}

		/// <summary>
		/// A test for CalculateAvailability.
		/// </summary>
		[TestMethod()]
		[DeploymentItem("TaskPlanner.BusinessLogic.dll")]
		public void CalculateAvailabilityTest()
		{
			Collection<UsualAvailability> usualAvailabilities = new Collection<UsualAvailability>();
			DateTime today = DateTime.Today;
			DateTime tomorrow = DateTime.Today.AddDays(1);
			UsualAvailability usualAvailability = new UsualAvailability();
			usualAvailability.Availability = 5;
			usualAvailability.DayOfWeek = today.DayOfWeek.ToString();
			usualAvailabilities.Add(usualAvailability);
			usualAvailability = new UsualAvailability();
			usualAvailability.Availability = 5;
			usualAvailability.DayOfWeek = tomorrow.DayOfWeek.ToString();
			usualAvailabilities.Add(usualAvailability);
			Collection<SpecificAvailability> specificAvailabilities = new Collection<SpecificAvailability>();
			SpecificAvailability specificAvailability = new SpecificAvailability();
			specificAvailability.Availability = 6;
			specificAvailability.Date = tomorrow;
			specificAvailabilities.Add(specificAvailability);
			Assert.AreEqual(5, TaskManager_Accessor.CalculateAvailability(usualAvailabilities, specificAvailabilities, today));
			Assert.AreEqual(6, TaskManager_Accessor.CalculateAvailability(usualAvailabilities, specificAvailabilities, tomorrow));
		}

		/// <summary>
		/// A test for CalculateOptimalDueDate.
		/// </summary>
		[TestMethod()]
		[DeploymentItem("TaskPlanner.BusinessLogic.dll")]
		public void CalculateOptimalDueDateTest()
		{
			Collection<UsualAvailability> usualAvailabilities = new Collection<UsualAvailability>();
			UsualAvailability usualAvailability = new UsualAvailability();
			usualAvailability.Availability = 4;
			usualAvailability.DayOfWeek = DateTime.Today.DayOfWeek.ToString();
			usualAvailabilities.Add(usualAvailability);
			Collection<SpecificAvailability> specificAvailabilities = new Collection<SpecificAvailability>();
			SpecificAvailability specificAvailability = new SpecificAvailability();
			specificAvailability.Availability = 4;
			specificAvailability.Date = DateTime.Today.AddDays(1);
			specificAvailabilities.Add(specificAvailability);
			Dictionary<DateTime, float> remainingAvailability = new Dictionary<DateTime, float>();
			float taskRemainingTime = 7;
			DateTime startValue = DateTime.Today;
			DateTime expected = DateTime.Today.AddDays(1);
			DateTime actual = TaskManager_Accessor.CalculateOptimalDueDate(
				usualAvailabilities,
				specificAvailabilities,
				remainingAvailability,
				taskRemainingTime,
				startValue);
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		/// A test for OnTaskChanged.
		/// </summary>
		[TestMethod()]
		[DeploymentItem("TaskPlanner.BusinessLogic.dll")]
		public void OnTaskChangedTest()
		{
			ITaskRepository taskRepository = new TaskRepository();
			IUsualAvailabilityRepository usualAvailabilityRepository = new UsualAvailabilityRepository();
			ISpecificAvailabilityRepository specificAvailabilityRepository = new SpecificAvailabilityRepository();
			TaskManager taskManager = new TaskManager(taskRepository, usualAvailabilityRepository, specificAvailabilityRepository);
			PrivateObject privateObject = new PrivateObject(taskManager);
			TaskManager_Accessor target = new TaskManager_Accessor(privateObject);
			TaskCorrectionRequestedEventArgs expectedE = new TaskCorrectionRequestedEventArgs();
			TaskCorrectionRequestedEventArgs actualE = null;
			ManualResetEvent eventWaiter = new ManualResetEvent(false);
			target.add_TaskCorrectionsRequested((sender, e) =>
			{
				actualE = e;
				eventWaiter.Set();
			});
			target.OnTaskChanged(expectedE);
			Assert.IsTrue(eventWaiter.WaitOne(5000));
			Assert.AreEqual(expectedE, actualE);
		}

		/// <summary>
		/// A test for ValidateAllTasks.
		/// </summary>
		[TestMethod()]
		public void ValidateAllTasksTest()
		{
			ITaskRepository taskRepository = new TaskRepository();
			IUsualAvailabilityRepository usualAvailabilityRepository = new UsualAvailabilityRepository();
			ISpecificAvailabilityRepository specificAvailabilityRepository = new SpecificAvailabilityRepository();
			TaskManager target = new TaskManager(taskRepository, usualAvailabilityRepository, specificAvailabilityRepository);
			DateTime availabilityDate = DateTime.Today;
			SpecificAvailability availability = new SpecificAvailability();
			availability.Date = availabilityDate;
			availability.Availability = 2;
			specificAvailabilityRepository.AddAvailability(availability);
			availability = new SpecificAvailability();
			availabilityDate = availabilityDate.AddDays(1);
			availability.Date = availabilityDate;
			availability.Availability = 6;
			specificAvailabilityRepository.AddAvailability(availability);
			Task task = new Task();
			task.Id = Guid.NewGuid();
			task.StartDate = DateTime.Today;
			task.DueDate = DateTime.Today.AddDays(1);
			task.EstimatedTime = 5;
			taskRepository.SaveTask(task);
			task = new Task();
			task.Id = Guid.NewGuid();
			task.StartDate = DateTime.Today;
			task.DueDate = DateTime.Today.AddDays(1);
			task.EstimatedTime = 2.5F;
			taskRepository.SaveTask(task);
			ManualResetEvent eventWaiter = new ManualResetEvent(false);
			target.TaskCorrectionsRequested += (sender, e) => eventWaiter.Set();
			target.ValidateAllTasks();
			Assert.IsFalse(eventWaiter.WaitOne(5000));
			availability = new SpecificAvailability();
			availability.Date = availabilityDate;
			availability.Availability = 2;
			specificAvailabilityRepository.AddAvailability(availability);
			availability = new SpecificAvailability();
			availability.Date = availabilityDate.AddDays(1);
			availability.Availability = 20;
			specificAvailabilityRepository.AddAvailability(availability);
			eventWaiter.Reset();
			target.TaskCorrectionsRequested += (sender, e) => eventWaiter.Set();
			target.ValidateAllTasks();
			Assert.IsTrue(eventWaiter.WaitOne(5000));
		}
		#endregion
	}
}
