﻿namespace TaskPlanner.BusinessLogic
{
	using System;
	using System.Collections.Generic;
	using System.Collections.ObjectModel;
	using System.Linq;
	using TaskPlanner.BusinessLogic.Interfaces;
	using TaskPlanner.Entities;
	using TaskPlanner.Repository.Interfaces;

	/// <summary>
	/// Represents class to perform operations with tasks.
	/// </summary>
	public class TaskManager : ITaskManager
	{
		#region Fields
		private ITaskRepository _taskRepository;
		private IUsualAvailabilityRepository _usualAvailabilityRepository;
		private ISpecificAvailabilityRepository _specificAvailabilityRepository;
		#endregion

		#region Constructors
		/// <summary>
		/// Initializes a new instance of the <see cref="TaskManager"/> class.
		/// </summary>
		/// <param name="taskRepository">The task repository.</param>
		/// <param name="usualAvailabilityRepository">The usual availability repository.</param>
		/// <param name="specificAvailabilityRepository">The specific availability repository.</param>
		public TaskManager(
			ITaskRepository taskRepository,
			IUsualAvailabilityRepository usualAvailabilityRepository,
			ISpecificAvailabilityRepository specificAvailabilityRepository)
		{
			if (taskRepository == null)
			{
				throw new ArgumentNullException("taskRepository");
			}
			if (usualAvailabilityRepository == null)
			{
				throw new ArgumentNullException("usualAvailabilityRepository");
			}
			if (specificAvailabilityRepository == null)
			{
				throw new ArgumentNullException("specificAvailabilityRepository");
			}
			_taskRepository = taskRepository;
			_usualAvailabilityRepository = usualAvailabilityRepository;
			_specificAvailabilityRepository = specificAvailabilityRepository;
		}
		#endregion

		#region Events
		/// <summary>
		/// Occurs when task is changed.
		/// </summary>
		public event EventHandler<TaskCorrectionRequestedEventArgs> TaskCorrectionsRequested;
		#endregion

		#region Methods
		/// <summary>
		/// Saves the task.
		/// </summary>
		/// <param name="task">The task to be saved.</param>
		public void SaveTask(Task task)
		{
			if (task == null)
			{
				throw new ArgumentNullException("task");
			}
			_taskRepository.SaveTask(task);
		}

		/// <summary>
		/// Deletes the sepcified task.
		/// </summary>
		/// <param name="task">The task to be deleted.</param>
		public void DeleteTask(Task task)
		{
			if (task == null)
			{
				throw new ArgumentNullException("task");
			}
			_taskRepository.DeleteTask(task);
		}

		/// <summary>
		/// Gets all tasks.
		/// </summary>
		/// <returns>The all tasks in repository.</returns>
		public Collection<Task> GetAllTasks()
		{
			return _taskRepository.GetAllTasks();
		}

		/// <summary>
		/// Gets the topmost level tasks and assigns their children recursively.
		/// </summary>
		/// <returns>Tasks tree.</returns>
		public Collection<Task> GetTasksTree()
		{
			return _taskRepository.GetTasksTree();
		}

		/// <summary>
		/// Creates the new task.
		/// </summary>
		/// <returns>Task with prefilled fields.</returns>
		public Task CreateNewTask()
		{
			Task result = new Task();
			result.Id = Guid.NewGuid();
			result.StartDate = DateTime.Today;
			result.DueDate = DateTime.Today;
			result.EndDate = DateTime.Today;
			return result;
		}

		/// <summary>
		/// Corrects task due to estimated time change.
		/// </summary>
		/// <param name="task">The task to be corrected.</param>
		/// <param name="isDueDateEdited">If set to <c>true</c> it means that due date was edited.</param>
		/// <returns>Task correction result.</returns>
		public TaskCorrectionResult ApplyEstimatedTimeChange(Task task, bool isDueDateEdited)
		{
			if (task == null)
			{
				throw new ArgumentNullException("task");
			}
			TaskCorrectionResult result = new TaskCorrectionResult();
			Collection<Task> activeTasks = _taskRepository.GetActiveTasks((item) => item.Id != task.Id, (item) => item.DueDate);
			Collection<UsualAvailability> usualAvailabilities = _usualAvailabilityRepository.GetUsualAvailabilities();
			Collection<SpecificAvailability> specificAvailabilities = _specificAvailabilityRepository.GetAvailabilities(DateTime.Today);
			Dictionary<DateTime, float> remainingAvailability =
				CalculateRemainingAvailability(activeTasks, usualAvailabilities, specificAvailabilities);
			float taskRemainingTime;
			if (isDueDateEdited)
			{
				taskRemainingTime = task.EstimatedTime - task.CompletedTime;
			}
			else
			{
				taskRemainingTime = (task.EstimatedTime - task.CompletedTime) * 2; // Business requirement.
			}
			DateTime currentDate = task.StartDate;
			DateTime optimalDueDate = CalculateOptimalDueDate(
				usualAvailabilities,
				specificAvailabilities,
				remainingAvailability,
				taskRemainingTime,
				currentDate);
			if (!isDueDateEdited)
			{
				result.IsCorrectionMade = true;
				result.CorrectionReason = TaskCorrectionReason.DueDateSuggestion;
				result.CorrectedTask = task.Clone();
				result.CorrectedTask.DueDate = optimalDueDate;
			}
			else if (optimalDueDate > task.DueDate)
			{
				result.IsCorrectionMade = true;
				result.CorrectionReason = TaskCorrectionReason.DueDateIsTooSmall;
				result.CorrectedTask = task.Clone();
				result.CorrectedTask.DueDate = optimalDueDate;
			}
			if (result.IsCorrectionMade)
			{
				TaskCorrectionRequestedEventArgs corectionDetails = new TaskCorrectionRequestedEventArgs();
				corectionDetails.CorrectedTask = result.CorrectedTask;
				corectionDetails.OriginalTask = task;
				corectionDetails.CorrectionReason = result.CorrectionReason;
				OnTaskChanged(corectionDetails);
			}
			return result;
		}

		/// <summary>
		/// Correctes the task due to completed percentage change.
		/// </summary>
		/// <param name="task">The task to be corrected.</param>
		/// <returns>Task correction result.</returns>
		public TaskCorrectionResult ApplyCompletedPercentageChange(Task task)
		{
			if (task == null)
			{
				throw new ArgumentNullException("task");
			}
			TaskCorrectionResult result = new TaskCorrectionResult();
			if (task.CompletedPercentage == 100)
			{
				result.IsCorrectionMade = true;
				result.CorrectionReason = TaskCorrectionReason.EndDateSuggestion;
				result.CorrectedTask = task.Clone();
				result.CorrectedTask.EndDate = DateTime.Today;
			}
			else
			{
				float expectedEstimatedTime = (task.CompletedTime * 100) / task.CompletedPercentage;
				if (task.EstimatedTime != expectedEstimatedTime)
				{
					result.IsCorrectionMade = true;
					result.CorrectionReason = TaskCorrectionReason.EstimatedTimeSuggestion;
					result.CorrectedTask = task.Clone();
					result.CorrectedTask.EstimatedTime = expectedEstimatedTime;
				}
			}
			if (result.IsCorrectionMade)
			{
				TaskCorrectionRequestedEventArgs corectionDetails = new TaskCorrectionRequestedEventArgs();
				corectionDetails.CorrectedTask = result.CorrectedTask;
				corectionDetails.OriginalTask = task;
				corectionDetails.CorrectionReason = result.CorrectionReason;
				OnTaskChanged(corectionDetails);
			}
			return result;
		}

		/// <summary>
		/// Corrects the task due to completed time change.
		/// </summary>
		/// <param name="task">The task to be corrected.</param>
		/// <returns>Task correction result.</returns>
		public TaskCorrectionResult ApplyCompletedTimeChange(Task task)
		{
			if (task == null)
			{
				throw new ArgumentNullException("task");
			}
			TaskCorrectionResult result = new TaskCorrectionResult();
			int expectedCompletedPercentage = (int)Math.Round((decimal)(100 * task.CompletedTime / task.EstimatedTime));
			if (task.CompletedPercentage != expectedCompletedPercentage)
			{
				result.IsCorrectionMade = true;
				result.CorrectionReason = TaskCorrectionReason.CompletedPercentageSuggestion;
				result.CorrectedTask = task.Clone();
				result.CorrectedTask.CompletedPercentage = expectedCompletedPercentage;
			}
			if (result.IsCorrectionMade)
			{
				TaskCorrectionRequestedEventArgs corectionDetails = new TaskCorrectionRequestedEventArgs();
				corectionDetails.CorrectedTask = result.CorrectedTask;
				corectionDetails.OriginalTask = task;
				corectionDetails.CorrectionReason = result.CorrectionReason;
				OnTaskChanged(corectionDetails);
			}
			return result;
		}

		/// <summary>
		/// Validates all tasks.
		/// </summary>
		public void ValidateAllTasks()
		{
			foreach (Task task in _taskRepository.GetActiveTasks(task => task.DueDate))
			{
				ApplyEstimatedTimeChange(task, true);
			}
		}

		/// <summary>
		/// Raises the <see cref="E:TaskChanged"/> event.
		/// </summary>
		/// <param name="e">The <see cref="TaskPlanner.BusinessLogic.Interfaces.TaskCorrectionRequestedEventArgs"/> instance containing the event data.</param>
		protected void OnTaskChanged(TaskCorrectionRequestedEventArgs e)
		{
			EventHandler<TaskCorrectionRequestedEventArgs> handler = TaskCorrectionsRequested;
			if (handler != null)
			{
				handler(this, e);
			}
		}

		/// <summary>
		/// Calculates the optimal due date.
		/// </summary>
		/// <param name="usualAvailabilities">The usual availabilities.</param>
		/// <param name="specificAvailabilities">The specific availabilities.</param>
		/// <param name="remainingAvailability">The remaining availability.</param>
		/// <param name="taskRemainingTime">The task remaining time.</param>
		/// <param name="startValue">The current date.</param>
		/// <returns>The optimal due date for task.</returns>
		private static DateTime CalculateOptimalDueDate(
			Collection<UsualAvailability> usualAvailabilities,
			Collection<SpecificAvailability> specificAvailabilities,
			Dictionary<DateTime, float> remainingAvailability,
			float taskRemainingTime,
			DateTime startValue)
		{
			DateTime optimalDueDate = startValue;
			while (taskRemainingTime > 0)
			{
				float availability;
				if (remainingAvailability.ContainsKey(startValue))
				{
					availability = remainingAvailability[startValue];
				}
				else
				{
					availability = CalculateAvailability(usualAvailabilities, specificAvailabilities, startValue);
				}
				taskRemainingTime -= availability;
				optimalDueDate = startValue;
				startValue = startValue.AddDays(1);
			}
			return optimalDueDate;
		}

		/// <summary>
		/// Calculates the remaining availability.
		/// </summary>
		/// <param name="tasks">The given tasks.</param>
		/// <param name="usualAvailabilities">The usual availabilities.</param>
		/// <param name="specificAvailabilities">The specific availabilities.</param>
		/// <returns>The availability after applying workload for all given tasks.</returns>
		private static Dictionary<DateTime, float> CalculateRemainingAvailability(
			Collection<Task> tasks,
			Collection<UsualAvailability> usualAvailabilities,
			Collection<SpecificAvailability> specificAvailabilities)
		{
			Dictionary<DateTime, float> result = new Dictionary<DateTime, float>();
			foreach (Task activeTask in tasks)
			{
				DateTime currentDate = DateTime.Today;
				float totalAvailableTimeForTask = 0;
				while (currentDate <= activeTask.DueDate)
				{
					if (!result.ContainsKey(currentDate))
					{
						float availability = CalculateAvailability(usualAvailabilities, specificAvailabilities, currentDate);
						result.Add(currentDate, availability);
					}
					totalAvailableTimeForTask += result[currentDate];
					currentDate = currentDate.AddDays(1);
				}
				currentDate = DateTime.Today;
				float taskRemainingTime = activeTask.EstimatedTime - activeTask.CompletedTime;
				while (currentDate <= activeTask.DueDate)
				{
					result[currentDate] -= result[currentDate] * taskRemainingTime / totalAvailableTimeForTask;
					currentDate = currentDate.AddDays(1);
				}
			}
			return result;
		}

		/// <summary>
		/// Calculates the availability.
		/// </summary>
		/// <param name="usualAvailabilities">The usual availabilities.</param>
		/// <param name="specificAvailabilities">The specific availabilities.</param>
		/// <param name="currentDate">The current date.</param>
		/// <returns>Specific availability for the date if it exists, otherwise usual availability.</returns>
		private static float CalculateAvailability(
			Collection<UsualAvailability> usualAvailabilities,
			Collection<SpecificAvailability> specificAvailabilities,
			DateTime currentDate)
		{
			float resul = 0;
			SpecificAvailability specificAvailability =
			(from item in specificAvailabilities
			 where item.Date == currentDate
			 select item).SingleOrDefault();
			if (specificAvailability == null)
			{
				UsualAvailability usualAvailability =
					(from item in usualAvailabilities
					 where item.DayOfWeek == currentDate.DayOfWeek.ToString()
					 select item).Single();
				resul = usualAvailability.Availability;
			}
			else
			{
				resul = specificAvailability.Availability;
			}
			return resul;
		}
		#endregion
	}
}
