﻿namespace TaskPlanner.Core
{
	using System;
	using Microsoft.Practices.Unity;

	/// <summary>
	/// The locator of services in application.
	/// </summary>
	public sealed class ServiceLocator : IDisposable
	{
		#region Fields
		private static ServiceLocator _instance = new ServiceLocator();
		private IUnityContainer _container;
		#endregion

		#region Constructors
		/// <summary>
		/// Prevents a default instance of the <see cref="ServiceLocator"/> class from being created.
		/// </summary>
		private ServiceLocator()
		{
			_container = new UnityContainer();
		}
		#endregion

		#region Destructor
		/// <summary>
		/// Finalizes an instance of the <see cref="ServiceLocator"/> class. Releases unmanaged resources and performs other cleanup operations before the
		/// <see cref="ServiceLocator"/> is reclaimed by garbage collection.
		/// </summary>
		~ServiceLocator()
		{
			Dispose(false);
		}
		#endregion

		#region Properties
		/// <summary>
		/// Gets the instance.
		/// </summary>
		/// <value>The instance.</value>
		public static ServiceLocator Instance
		{
			get
			{
				return _instance;
			}
		}
		#endregion

		#region Methods
		/// <summary>
		/// Registers the type.
		/// </summary>
		/// <param name="baseType">The base type.</param>
		/// <param name="implementation">The implementation of the base type.</param>
		public void RegisterType(Type baseType, Type implementation)
		{
			_container.RegisterType(baseType, implementation);
		}

		/// <summary>
		/// Registers the type.
		/// </summary>
		/// <param name="baseType">The base type.</param>
		/// <param name="implementation">The implementation of the base type.</param>
		/// <param name="injectionMembers">The injection members.</param>
		public void RegisterType(Type baseType, Type implementation, params InjectionMember[] injectionMembers)
		{
			_container.RegisterType(baseType, implementation, injectionMembers);
		}

		/// <summary>
		/// Registers the type.
		/// </summary>
		/// <param name="implementation">The implementation.</param>
		public void RegisterType(Type implementation)
		{
			_container.RegisterType(implementation);
		}

		/// <summary>
		/// Resolves this instance.
		/// </summary>
		/// <typeparam name="T">The type to be resolved.</typeparam>
		/// <returns>Implementation of base type passed as parameter.</returns>
		public T Resolve<T>()
		{
			return _container.Resolve<T>();
		}

		/// <summary>
		/// Adds the extension.
		/// </summary>
		/// <param name="extension">The extension.</param>
		public void AddExtension(UnityContainerExtension extension)
		{
			_container = _container.AddExtension(extension);
		}

		/// <summary>
		/// Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources.
		/// </summary>
		public void Dispose()
		{
			Dispose(true);
			GC.SuppressFinalize(this);
		}

		/// <summary>
		/// Releases unmanaged and - optionally - managed resources.
		/// </summary>
		/// <param name="disposing"><c>True</c> to release both managed and unmanaged resources; <c>false</c> to release only unmanaged resources.</param>
		private void Dispose(bool disposing)
		{
			if (disposing)
			{
				_container.Dispose();
				_container = null;
			}
		}
		#endregion
	}
}
