﻿namespace TaskPlanner.Repository.Test
{
	using System;
	using System.Data.SqlServerCe;
	using System.IO;
	using Microsoft.Practices.Unity;
	using Microsoft.VisualStudio.TestTools.UnitTesting;
	using TaskPlanner.Core;

	/// <summary>
	/// This is a test class for DataHelperTest and is intended
	/// to contain all DataHelperTest Unit Tests.
	/// </summary>
	[TestClass()]
	public class DataHelperTest
	{
		#region Fields
		private static IUnityContainer _container;
		private TestContext _testContextInstance;
		#endregion

		#region Properties
		/// <summary>
		/// Gets or sets the test context which provides
		/// information about and functionality for the current test run.
		/// </summary>
		/// <value>The test context.</value>
		public TestContext TestContext
		{
			get
			{
				return _testContextInstance;
			}

			set
			{
				_testContextInstance = value;
			}
		}
		#endregion

		#region Methods
		/// <summary>
		/// Initializes this instance.
		/// </summary>
		/// <param name="testContext">The test context.</param>
		[ClassInitialize]
		public static void Initialize(TestContext testContext)
		{
			_container = new UnityContainer();
			_container.RegisterType<ILogHelper, TestsLogHelper>();
		}

		/// <summary>
		/// Teardowns this instance.
		/// </summary>
		[ClassCleanup]
		public static void Teardown()
		{
			_container.Dispose();
			_container = null;
		}

		/// <summary>
		/// A test for DataHelper Constructor.
		/// </summary>
		[TestMethod()]
		public void DataHelperConstructorTest()
		{
			LogHelper logHelper = null;
			Utils.AssertException(() => new DataHelper(logHelper), typeof(ArgumentNullException));
		}

		/// <summary>
		/// A test for ApplyPatch.
		/// </summary>
		[TestMethod()]
		[DeploymentItem("TaskPlanner.Repository.dll")]
		public void ApplyPatchTest()
		{
			DataHelper_Accessor target = new DataHelper_Accessor(_container.Resolve<ILogHelper>());
			target.UpdateDatabaseSchema();
			string expectedTableName = "Test";
			string databasePatch = string.Concat("CREATE TABLE [", expectedTableName, "]([Id] uniqueidentifier NOT NULL)");
			DataHelper_Accessor.ApplyPatch(databasePatch);
			using (SqlCeConnection connection = Utils.CreateConnection())
			{
				using (SqlCeCommand command = connection.CreateCommand())
				{
					command.CommandText = string.Concat("select TABLE_NAME from INFORMATION_SCHEMA.TABLES where TABLE_NAME = '", expectedTableName, "'");
					connection.Open();
					string actualTableName = command.ExecuteScalar() as string;
					Assert.AreEqual(expectedTableName, actualTableName);
				}
			}
		}

		/// <summary>
		/// A test for CreateConnection.
		/// </summary>
		[TestMethod()]
		[DeploymentItem("TaskPlanner.Repository.dll")]
		public void CreateConnectionTest()
		{
			SqlCeConnection expected = Utils.CreateConnection();
			SqlCeConnection actual;
			actual = DataHelper_Accessor.CreateConnection();
			Assert.AreEqual(expected.ConnectionString, actual.ConnectionString);
		}

		/// <summary>
		/// A test for DefineDatabaseVersion.
		/// </summary>
		[TestMethod()]
		[DeploymentItem("TaskPlanner.Repository.dll")]
		public void DefineDatabaseVersionTest()
		{
			if (File.Exists(Utils.DataSource))
			{
				File.Delete(Utils.DataSource);
			}
			string expected = "0.0.0.0";
			string actual;
			actual = DataHelper_Accessor.DefineDatabaseVersion();
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		/// A test for EnsureDatabaseHasLatestSchema.
		/// </summary>
		[TestMethod()]
		public void EnsureDatabaseHasLatestSchemaTest()
		{
			if (File.Exists(Utils.DataSource))
			{
				File.Delete(Utils.DataSource);
			}
			DataHelper target = new DataHelper(_container.Resolve<ILogHelper>());
			target.EnsureDatabaseHasLatestSchema();
			using (SqlCeConnection connection = Utils.CreateConnection())
			{
				using (SqlCeCommand command = connection.CreateCommand())
				{
					command.CommandText = "select PropertyValue from DatabaseConfiguration where PropertyName = 'DatabaseVersion'";
					connection.Open();
					string databaseVersion = command.ExecuteScalar() as string;
					Assert.AreEqual("1.0.3.0", databaseVersion);
				}
			}
		}

		/// <summary>
		/// A test for GetDatabasePatch.
		/// </summary>
		[TestMethod()]
		[DeploymentItem("TaskPlanner.Repository.dll")]
		public void GetDatabasePatchTest()
		{
			string databaseVersion = "0.0.0.0";
			string actual;
			actual = DataHelper_Accessor.GetDatabasePatch(databaseVersion);
			Assert.AreNotEqual(string.Empty, actual);
		}

		/// <summary>
		/// A test for GetResourceAsString.
		/// </summary>
		[TestMethod()]
		[DeploymentItem("TaskPlanner.Repository.dll")]
		public void GetResourceAsStringTest()
		{
			string resourceName = "Scripts.GetDatabaseVersion.sqlce";
			string expected = "select PropertyValue\r\nfrom DatabaseConfiguration\r\nwhere PropertyName = 'DatabaseVersion'\r\n";
			string actual;
			actual = DataHelper_Accessor.GetResourceAsString(resourceName);
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		/// A test for UpdateDatabaseSchema.
		/// </summary>
		[TestMethod()]
		[DeploymentItem("TaskPlanner.Repository.dll")]
		public void UpdateDatabaseSchemaTest()
		{
			if (File.Exists(Utils.DataSource))
			{
				File.Delete(Utils.DataSource);
			}
			DataHelper_Accessor target = new DataHelper_Accessor(_container.Resolve<ILogHelper>());
			target.UpdateDatabaseSchema();
			using (SqlCeConnection connection = Utils.CreateConnection())
			{
				using (SqlCeCommand command = connection.CreateCommand())
				{
					command.CommandText = "select PropertyValue from DatabaseConfiguration where PropertyName = 'DatabaseVersion'";
					connection.Open();
					string databaseVersion = command.ExecuteScalar() as string;
					Assert.AreEqual("1.0.3.0", databaseVersion);
				}
			}
		}

		/// <summary>
		/// A test for DataSource.
		/// </summary>
		[TestMethod()]
		[DeploymentItem("TaskPlanner.Repository.dll")]
		public void DataSourceTest()
		{
			string actual;
			actual = DataHelper_Accessor.DataSource;
			string expected = Utils.DataSource;
			Assert.AreEqual(expected, actual);
		}
		#endregion
	}
}
