﻿namespace TaskPlanner.Repository.Test
{
	using System;
	using System.Collections.ObjectModel;
	using System.IO;
	using System.Linq;
	using Microsoft.VisualStudio.TestTools.UnitTesting;
	using TaskPlanner.Entities;

	/// <summary>
	/// This is a test class for TaskRepositoryTest and is intended
	/// to contain all TaskRepositoryTest Unit Tests.
	/// </summary>
	[TestClass()]
	public class TaskRepositoryTest
	{
		#region Fields
		private TestContext _testContextInstance;
		#endregion

		#region Properties
		/// <summary>
		/// Gets or sets the test context which provides
		/// information about and functionality for the current test run.
		/// </summary>
		/// <value>The test context.</value>
		public TestContext TestContext
		{
			get
			{
				return _testContextInstance;
			}

			set
			{
				_testContextInstance = value;
			}
		}
		#endregion

		#region Methods
		/// <summary>
		/// Initializes the every test.
		/// </summary>
		[TestInitialize]
		public void InitializeTest()
		{
			if (File.Exists(Utils.DataSource))
			{
				File.Delete(Utils.DataSource);
			}
			DataHelper dataHelper = new DataHelper(new TestsLogHelper());
			dataHelper.EnsureDatabaseHasLatestSchema();
		}

		/// <summary>
		/// A test for TaskRepository Constructor.
		/// </summary>
		[TestMethod()]
		public void TaskRepositoryConstructorTest()
		{
			TaskRepository target = new TaskRepository(); // Just to ensure that there is no exceptions.
		}

		/// <summary>
		/// A test for GetAllTasks.
		/// </summary>
		[TestMethod()]
		public void GetAllTasksTest()
		{
			TaskRepository target = new TaskRepository();
			Task task = new Task();
			task.Id = Guid.NewGuid();
			string expectedTaskName = Guid.NewGuid().ToString();
			task.TaskName = expectedTaskName;
			task.StartDate = DateTime.Today;
			task.EndDate = DateTime.Today;
			task.DueDate = DateTime.Today;
			task.Category = Guid.NewGuid().ToString();
			target.SaveTask(task);
			Collection<Task> actual;
			actual = target.GetAllTasks();
			Assert.AreNotEqual(0, actual.Count);
			Task taskInDatabase =
				(from item in actual
				 where item.Id == task.Id
				 select item).Single();
			Assert.IsNotNull(taskInDatabase);
		}

		/// <summary>
		/// A test for SaveTask.
		/// </summary>
		[TestMethod()]
		public void SaveTaskTest()
		{
			TaskRepository target = new TaskRepository();
			Task task = null;
			Utils.AssertException(() => target.SaveTask(task), typeof(ArgumentNullException));
			task = new Task();
			task.Id = Guid.NewGuid();
			string expectedTaskName = Guid.NewGuid().ToString();
			task.TaskName = expectedTaskName;
			task.StartDate = DateTime.Today;
			task.EndDate = DateTime.Today;
			task.DueDate = DateTime.Today;
			task.Category = Guid.NewGuid().ToString();
			target.SaveTask(task);
			string actualTaskName =
				(from item in target.GetAllTasks()
				 where item.Id == task.Id
				 select item.TaskName).Single();
			Assert.AreEqual(expectedTaskName, actualTaskName);
			expectedTaskName = Guid.NewGuid().ToString();
			task.TaskName = expectedTaskName;
			target.SaveTask(task);
			actualTaskName =
				(from item in target.GetAllTasks()
				 where item.Id == task.Id
				 select item.TaskName).Single();
			Assert.AreEqual(expectedTaskName, actualTaskName);
		}

		/// <summary>
		/// A test for GetTasksTree.
		/// </summary>
		[TestMethod()]
		public void GetTasksTreeTest()
		{
			TaskRepository target = new TaskRepository();
			Task parentTask = new Task();
			Guid parentTaskId = Guid.NewGuid();
			parentTask.Id = parentTaskId;
			parentTask.TaskName = Guid.NewGuid().ToString();
			parentTask.StartDate = DateTime.Today;
			parentTask.EndDate = DateTime.Today;
			parentTask.DueDate = DateTime.Today;
			parentTask.Category = Guid.NewGuid().ToString();
			target.SaveTask(parentTask);
			Task childTask = new Task();
			childTask.Id = Guid.NewGuid();
			childTask.TaskName = Guid.NewGuid().ToString();
			childTask.StartDate = DateTime.Today;
			childTask.EndDate = DateTime.Today;
			childTask.DueDate = DateTime.Today;
			childTask.Category = Guid.NewGuid().ToString();
			childTask.Parent = parentTask;
			target.SaveTask(childTask);
			parentTask = childTask;
			childTask = new Task();
			childTask.Id = Guid.NewGuid();
			childTask.TaskName = Guid.NewGuid().ToString();
			childTask.StartDate = DateTime.Today;
			childTask.EndDate = DateTime.Today;
			childTask.DueDate = DateTime.Today;
			childTask.Category = Guid.NewGuid().ToString();
			childTask.Parent = parentTask;
			target.SaveTask(childTask);
			Collection<Task> tasksTree = target.GetTasksTree();
			Assert.AreEqual(1, tasksTree.Count);
			Task actualParentTask = tasksTree[0];
			Assert.IsNotNull(actualParentTask.Children);
			Assert.AreEqual(1, actualParentTask.Children.Count);
			Task actualSubParentTask = actualParentTask.Children[0];
			Assert.IsNotNull(actualParentTask.Children);
			Assert.AreEqual(1, actualSubParentTask.Children.Count);
		}

		/// <summary>
		/// A test for GetActiveTasks.
		/// </summary>
		[TestMethod()]
		public void GetActiveTasksWithFilterAndOrderTest()
		{
			TaskRepository target = new TaskRepository();
			Task notCompletedTask = new Task();
			string expectedTaskName = Guid.NewGuid().ToString();
			notCompletedTask.TaskName = expectedTaskName;
			notCompletedTask.StartDate = DateTime.Today;
			notCompletedTask.EndDate = DateTime.Today;
			notCompletedTask.DueDate = DateTime.Today;
			notCompletedTask.Category = Guid.NewGuid().ToString();
			notCompletedTask.Id = Guid.NewGuid();
			target.SaveTask(notCompletedTask);
			Task completedTask = new Task();
			completedTask.CompletedPercentage = 100;
			completedTask.StartDate = DateTime.Today;
			completedTask.EndDate = DateTime.Today;
			completedTask.DueDate = DateTime.Today;
			completedTask.Category = Guid.NewGuid().ToString();
			completedTask.TaskName = Guid.NewGuid().ToString();
			completedTask.Id = Guid.NewGuid();
			target.SaveTask(completedTask);
			Collection<Task> activeTasks = target.GetActiveTasks((item) => true, (item) => 1);
			Assert.IsNotNull(activeTasks);
			Assert.AreEqual(1, activeTasks.Count);
			Assert.IsNotNull(activeTasks[0]);
			Assert.AreEqual(expectedTaskName, activeTasks[0].TaskName);
		}

		/// <summary>
		/// A test for GetActiveTasks.
		/// </summary>
		[TestMethod()]
		public void GetActiveTasksWithOrderTest()
		{
			TaskRepository target = new TaskRepository();
			Task notCompletedTask = new Task();
			string expectedTaskName = Guid.NewGuid().ToString();
			notCompletedTask.TaskName = expectedTaskName;
			notCompletedTask.StartDate = DateTime.Today;
			notCompletedTask.EndDate = DateTime.Today;
			notCompletedTask.DueDate = DateTime.Today;
			notCompletedTask.Category = Guid.NewGuid().ToString();
			notCompletedTask.Id = Guid.NewGuid();
			target.SaveTask(notCompletedTask);
			Task completedTask = new Task();
			completedTask.CompletedPercentage = 100;
			completedTask.StartDate = DateTime.Today;
			completedTask.EndDate = DateTime.Today;
			completedTask.DueDate = DateTime.Today;
			completedTask.Category = Guid.NewGuid().ToString();
			completedTask.TaskName = Guid.NewGuid().ToString();
			completedTask.Id = Guid.NewGuid();
			target.SaveTask(completedTask);
			Collection<Task> activeTasks = target.GetActiveTasks((item) => 1);
			Assert.IsNotNull(activeTasks);
			Assert.AreEqual(1, activeTasks.Count);
			Assert.IsNotNull(activeTasks[0]);
			Assert.AreEqual(expectedTaskName, activeTasks[0].TaskName);
		}

		/// <summary>
		/// A test for DeleteTask.
		/// </summary>
		[TestMethod()]
		public void DeleteTaskTest()
		{
			TaskRepository target = new TaskRepository();
			Task task = null;
			Utils.AssertException(() => target.SaveTask(task), typeof(ArgumentNullException));
			task = new Task();
			task.Id = Guid.NewGuid();
			task.TaskName = Guid.NewGuid().ToString();
			task.StartDate = DateTime.Today;
			task.EndDate = DateTime.Today;
			task.DueDate = DateTime.Today;
			task.Category = Guid.NewGuid().ToString();
			target.SaveTask(task);
			target.DeleteTask(task);
			Task taskInDatabase =
				(from item in target.GetAllTasks()
				 where item.Id == task.Id
				 select item).SingleOrDefault();
			Assert.IsNull(taskInDatabase);
			Task parentTask = new Task();
			parentTask.Id = Guid.NewGuid();
			parentTask.TaskName = Guid.NewGuid().ToString();
			parentTask.StartDate = DateTime.Today;
			parentTask.EndDate = DateTime.Today;
			parentTask.DueDate = DateTime.Today;
			parentTask.Category = Guid.NewGuid().ToString();
			target.SaveTask(parentTask);
			Task childTask = new Task();
			childTask.Id = Guid.NewGuid();
			childTask.TaskName = Guid.NewGuid().ToString();
			childTask.StartDate = DateTime.Today;
			childTask.EndDate = DateTime.Today;
			childTask.DueDate = DateTime.Today;
			childTask.Category = Guid.NewGuid().ToString();
			childTask.ParentId = parentTask.Id;
			target.SaveTask(childTask);
			Task anotherTask = new Task();
			anotherTask.Id = Guid.NewGuid();
			anotherTask.TaskName = Guid.NewGuid().ToString();
			anotherTask.StartDate = DateTime.Today;
			anotherTask.EndDate = DateTime.Today;
			anotherTask.DueDate = DateTime.Today;
			anotherTask.Category = Guid.NewGuid().ToString();
			target.SaveTask(anotherTask);
			target.DeleteTask(parentTask);
			taskInDatabase =
				(from item in target.GetAllTasks()
				 where item.Id == parentTask.Id
				 select item).SingleOrDefault();
			Assert.IsNull(taskInDatabase);
			taskInDatabase =
				(from item in target.GetAllTasks()
				 where item.Id == childTask.Id
				 select item).SingleOrDefault();
			Assert.IsNull(taskInDatabase);
			taskInDatabase =
				(from item in target.GetAllTasks()
				 where item.Id == anotherTask.Id
				 select item).SingleOrDefault();
			Assert.IsNotNull(taskInDatabase);
		}
		#endregion
	}
}
