﻿namespace TaskPlanner.Repository
{
	using System;
	using System.Configuration;
	using System.Data;
	using System.Data.EntityClient;
	using System.Data.SqlClient;
	using System.Data.SqlServerCe;
	using System.Globalization;
	using System.IO;
	using System.Reflection;
	using System.Text.RegularExpressions;
	using TaskPlanner.Core;
	using TaskPlanner.Repository.Interfaces;

	/// <summary>
	/// Represents a helper to work with data.
	/// </summary>
	public class DataHelper : IDataHelper
	{
		#region Constants
		private const string NullDatabase = "0.0.0.0";
		private const string ScriptsNamespace = "Scripts.";
		#endregion

		#region Fields
		private ILogHelper _logHelper;
		#endregion

		#region Constructors
		/// <summary>
		/// Initializes a new instance of the <see cref="DataHelper"/> class.
		/// </summary>
		/// <param name="logHelper">The log helper.</param>
		public DataHelper(ILogHelper logHelper)
		{
			if (logHelper == null)
			{
				throw new ArgumentNullException("logHelper");
			}
			_logHelper = logHelper;
		}
		#endregion

		#region Properties
		/// <summary>
		/// Gets the data source.
		/// </summary>
		/// <value>The data source.</value>
		private static string DataSource
		{
			get
			{
				using (EntityConnection entityConnection = new EntityConnection(ConfigurationManager.ConnectionStrings["TaskPlannerEntities"].ConnectionString))
				{
					return entityConnection.DataSource;
				}
			}
		}
		#endregion

		#region Methods
		/// <summary>
		/// Ensures the database has latest schema.
		/// </summary>
		public void EnsureDatabaseHasLatestSchema()
		{
			UpdateDatabaseSchema();
		}

		/// <summary>
		/// Defines the database version.
		/// </summary>
		/// <returns>Database version.</returns>
		private static string DefineDatabaseVersion()
		{
			string result = null;
			if (!File.Exists(DataSource))
			{
				result = NullDatabase;
			}
			else
			{
				using (SqlCeConnection connection = CreateConnection())
				{
					using (SqlCeCommand command = connection.CreateCommand())
					{
						command.CommandText = GetResourceAsString(string.Concat(ScriptsNamespace, "GetDatabaseVersion.sqlce"));
						if (connection.State != ConnectionState.Open)
						{
							connection.Open();
						}
						result = command.ExecuteScalar() as string;
					}
				}
			}
			return result;
		}

		/// <summary>
		/// Creates the connection.
		/// </summary>
		/// <returns>Sql connection.</returns>
		private static SqlCeConnection CreateConnection()
		{
			SqlConnectionStringBuilder connectionStringBuilder = new SqlConnectionStringBuilder();
			connectionStringBuilder.DataSource = DataSource;
			return new SqlCeConnection(connectionStringBuilder.ConnectionString);
		}

		/// <summary>
		/// Gets the resource as string.
		/// </summary>
		/// <param name="resourceName">Name of the resource.</param>
		/// <returns>Resource as string.</returns>
		private static string GetResourceAsString(string resourceName)
		{
			string result = string.Empty;
			Assembly executingAssembly = Assembly.GetExecutingAssembly();
			Stream resourceStream = executingAssembly.GetManifestResourceStream(typeof(DataHelper), resourceName);
			if (resourceStream != null)
			{
				using (StreamReader resourceReader = new StreamReader(resourceStream))
				{
					result = resourceReader.ReadToEnd();
				}
			}
			return result;
		}

		/// <summary>
		/// Gets the database patch.
		/// </summary>
		/// <param name="databaseVersion">The database version.</param>
		/// <returns>Database patch.</returns>
		private static string GetDatabasePatch(string databaseVersion)
		{
			return GetResourceAsString(string.Concat(ScriptsNamespace, "Patches.", databaseVersion, ".sqlce"));
		}

		/// <summary>
		/// Applies the patch.
		/// </summary>
		/// <param name="databasePatch">The database patch.</param>
		private static void ApplyPatch(string databasePatch)
		{
			using (SqlCeConnection connection = CreateConnection())
			{
				foreach (string singleCommand in Regex.Split(databasePatch, @"\s+go\s+", RegexOptions.IgnoreCase | RegexOptions.CultureInvariant))
				{
					if (!string.IsNullOrWhiteSpace(singleCommand))
					{
						using (SqlCeCommand command = connection.CreateCommand())
						{
							command.CommandText = singleCommand;
							if (connection.State != ConnectionState.Open)
							{
								connection.Open();
							}
							command.ExecuteNonQuery();
						}
					}
				}
			}
		}

		/// <summary>
		/// Updates the database schema.
		/// </summary>
		private void UpdateDatabaseSchema()
		{
			_logHelper.WriteInfo("Starting updating database schema.");
			string databaseVersion = DefineDatabaseVersion();
			if (databaseVersion.Equals(NullDatabase, StringComparison.OrdinalIgnoreCase))
			{
				Assembly executingAssembly = Assembly.GetExecutingAssembly();
				using (SqlCeConnection connection = CreateConnection())
				{
					using (Stream resourceStream = executingAssembly.GetManifestResourceStream(typeof(DataHelper), "Model.database.sdf"))
					{
						using (FileStream databaseFile = new FileStream(DataSource, FileMode.Create, FileAccess.Write))
						{
							resourceStream.CopyTo(databaseFile);
						}
					}
				}
			}
			_logHelper.WriteInfo(CultureInfo.InvariantCulture, "Database version is '{0}'", databaseVersion);
			string databasePatch = GetDatabasePatch(databaseVersion);
			while (!string.IsNullOrWhiteSpace(databasePatch))
			{
				ApplyPatch(databasePatch);
				databaseVersion = DefineDatabaseVersion();
				_logHelper.WriteInfo(CultureInfo.InvariantCulture, "Database version is '{0}'", databaseVersion);
				databasePatch = GetDatabasePatch(databaseVersion);
			}
			_logHelper.WriteInfo("Database schema is updated.");
		}
		#endregion
	}
}
