﻿namespace TaskPlanner.Repository
{
	using System;
	using System.Collections.Generic;
	using System.Collections.ObjectModel;
	using System.Data.Objects;
	using System.Linq;
	using System.Linq.Expressions;
	using TaskPlanner.Entities;
	using TaskPlanner.Repository.Interfaces;
	using TaskPlanner.Repository.Model;

	/// <summary>
	/// Represents the operations with tasks.
	/// </summary>
	public class TaskRepository : ITaskRepository
	{
		#region Methods
		/// <summary>
		/// Saves the task.
		/// </summary>
		/// <param name="task">The task to be saved.</param>
		public void SaveTask(Task task)
		{
			if (task == null)
			{
				throw new ArgumentNullException("task");
			}
			using (TaskPlannerDataContext dataContext = new TaskPlannerDataContext())
			{
				if (task.Parent != null)
				{
					Task parentTask =
					(from item in dataContext.Tasks
					 where item.Id == task.Parent.Id
					 select item).Single();
					ObjectStateEntry entry = dataContext.ObjectStateManager.GetObjectStateEntry(parentTask);
					entry.ApplyCurrentValues(task.Parent);
					task.Parent = entry.Entity as Task;
				}
				Task existentTask =
					(from item in dataContext.Tasks
					 where item.Id == task.Id
					 select item).SingleOrDefault();
				if (existentTask != null)
				{
					ObjectStateEntry entry = dataContext.ObjectStateManager.GetObjectStateEntry(existentTask);
					entry.ApplyCurrentValues(task);
				}
				else
				{
					dataContext.Tasks.AddObject(task);
				}
				dataContext.SaveChanges();
			}
		}

		/// <summary>
		/// Gets all tasks.
		/// </summary>
		/// <returns>The all tasks in repository.</returns>
		public Collection<Task> GetAllTasks()
		{
			using (TaskPlannerDataContext dataContext = new TaskPlannerDataContext())
			{
				Collection<Task> result = new Collection<Task>(dataContext.Tasks.ToList());
				return result;
			}
		}

		/// <summary>
		/// Gets the topmost level tasks and assigns their children recursively.
		/// </summary>
		/// <returns>Tasks tree.</returns>
		public Collection<Task> GetTasksTree()
		{
			using (TaskPlannerDataContext dataContext = new TaskPlannerDataContext())
			{
				List<Task> queryResult = dataContext.Tasks.ToList();
				queryResult =
					(from item in queryResult
					 where item.Parent == null
					 select item).ToList();
				return new Collection<Task>(queryResult);
			}
		}

		/// <summary>
		/// Gets the active tasks.
		/// </summary>
		/// <typeparam name="TOrder">The type of the order.</typeparam>
		/// <param name="filter">The filter.</param>
		/// <param name="order">The order.</param>
		/// <returns>Active tasks.</returns>
		public Collection<Task> GetActiveTasks<TOrder>(
			Expression<Func<Task, bool>> filter,
			Expression<Func<Task, TOrder>> order)
		{
			using (TaskPlannerDataContext dataContext = new TaskPlannerDataContext())
			{
				return new Collection<Task>(
					(from item in dataContext.Tasks.Where(filter).OrderBy(order)
					 where item.CompletedPercentage < 100
					 select item).ToList());
			}
		}

		/// <summary>
		/// Gets the active tasks.
		/// </summary>
		/// <typeparam name="TOrder">The type of the order.</typeparam>
		/// <param name="order">The order.</param>
		/// <returns>Active tasks in specified order.</returns>
		public Collection<Task> GetActiveTasks<TOrder>(Expression<Func<Task, TOrder>> order)
		{
			using (TaskPlannerDataContext dataContext = new TaskPlannerDataContext())
			{
				return new Collection<Task>(
					(from item in dataContext.Tasks.OrderBy(order)
					 where item.CompletedPercentage < 100
					 select item).ToList());
			}
		}

		/// <summary>
		/// Deletes the task.
		/// </summary>
		/// <param name="task">The task to be deleted.</param>
		public void DeleteTask(Task task)
		{
			if (task == null)
			{
				throw new ArgumentNullException("task");
			}
			using (TaskPlannerDataContext dataContext = new TaskPlannerDataContext())
			{
				dataContext.Tasks.Attach(task);
				dataContext.Tasks.DeleteObject(task);
				List<Guid> parentsIds = new List<Guid>();
				parentsIds.Add(task.Id);
				while (parentsIds.Count != 0)
				{
					List<Task> children =
						(from item in dataContext.Tasks
						 where item.ParentId != null && parentsIds.Contains(item.ParentId.Value)
						 select item).ToList();
					parentsIds = new List<Guid>();
					foreach (Task childTask in children)
					{
						parentsIds.Add(childTask.Id);
						dataContext.Tasks.DeleteObject(childTask);
					}
				}
				dataContext.SaveChanges();
			}
		}
		#endregion
	}
}
