﻿namespace TaskPlanner
{
	using System;
	using System.Windows.Input;

	/// <summary>
	/// Represents a command.
	/// </summary>
	public class Command : ICommand
	{
		#region Fields
		private Action<object> _executeAction;
		private Predicate<object> _canExecutePredicate;
		#endregion

		#region Constructors
		/// <summary>
		/// Initializes a new instance of the <see cref="Command"/> class.
		/// </summary>
		/// <param name="executeAction">The action, which will be used when command executes.</param>
		/// <param name="canExecutePredicate">The predicate to check, whether command can be executed.</param>
		public Command(Action<object> executeAction, Predicate<object> canExecutePredicate)
		{
			if (executeAction == null)
			{
				throw new ArgumentNullException("executeAction");
			}
			_executeAction = executeAction;
			_canExecutePredicate = canExecutePredicate;
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="Command"/> class.
		/// </summary>
		/// <param name="executeAction">The action, which will be used when command executes.</param>
		public Command(Action<object> executeAction)
			: this(executeAction, null)
		{
		}
		#endregion

		#region Events
		/// <summary>
		/// Occurs when changes occur that affect whether or not the command should execute.
		/// </summary>
		public event EventHandler CanExecuteChanged
		{
			add
			{
				CommandManager.RequerySuggested += value;
			}

			remove
			{
				CommandManager.RequerySuggested -= value;
			}
		}
		#endregion

		#region Methods
		/// <summary>
		/// Defines the method to be called when the command is invoked.
		/// </summary>
		/// <param name="parameter">Data used by the command.  If the command does not require data to be passed, this object can be set to null.</param>
		public void Execute(object parameter)
		{
			_executeAction(parameter);
		}

		/// <summary>
		/// Defines the method that determines whether the command can execute in its current state.
		/// </summary>
		/// <param name="parameter">Data used by the command.  If the command does not require data to be passed, this object can be set to null.</param>
		/// <returns>
		/// True if this command can be executed; otherwise, false.
		/// </returns>
		public bool CanExecute(object parameter)
		{
			return _canExecutePredicate == null ? true : _canExecutePredicate(parameter);
		}
		#endregion
	}
}
