﻿namespace TaskPlanner.ViewModels
{
	using System;
	using System.Collections.Generic;
	using System.Collections.ObjectModel;
	using System.Globalization;
	using System.Linq;
	using System.Windows.Input;
	using TaskPlanner.BusinessLogic.Interfaces;
	using TaskPlanner.Entities;

	/// <summary>
	/// Represents a view model to edit availability.
	/// </summary>
	public class EditAvailabilityViewModel : ViewModelBase
	{
		#region Fields
		private UsualAvailabilityViewModel _usualAvailability;
		private IAvailabilityManager _availabilityManager;
		private ITaskManager _taskManager;
		private ICategoryManager _categoryManager;
		private ICommand _saveCommand;
		private SpecificAvailabilityListViewModel _specificAvailabilities;
		private SpecificAvailabilityViewModel _newSpecificAvailability;
		private ICommand _addAvailabilityCommand;
		#endregion

		#region Constructors
		/// <summary>
		/// Initializes a new instance of the <see cref="EditAvailabilityViewModel"/> class.
		/// </summary>
		/// <param name="availabilityManager">The availability manager.</param>
		/// <param name="taskManager">The task manager.</param>
		/// <param name="categoryManager">The category manager.</param>
		public EditAvailabilityViewModel(IAvailabilityManager availabilityManager, ITaskManager taskManager, ICategoryManager categoryManager)
		{
			_availabilityManager = availabilityManager;
			_taskManager = taskManager;
			_categoryManager = categoryManager;
		}
		#endregion

		#region Properties
		/// <summary>
		/// Gets the usual availability.
		/// </summary>
		/// <value>The usual availability.</value>
		public UsualAvailabilityViewModel UsualAvailability
		{
			get
			{
				if (_usualAvailability == null)
				{
					Collection<UsualAvailability> usualAvailability = _availabilityManager.GetUsualAvailabilities();
					List<DayOfWeek> order = new List<DayOfWeek>();
					order.Add(DayOfWeek.Sunday);
					order.Add(DayOfWeek.Monday);
					order.Add(DayOfWeek.Tuesday);
					order.Add(DayOfWeek.Wednesday);
					order.Add(DayOfWeek.Thursday);
					order.Add(DayOfWeek.Friday);
					order.Add(DayOfWeek.Saturday);
					DateTimeFormatInfo currentDateFormatInfo = CultureInfo.CurrentUICulture.DateTimeFormat;
					DateTimeFormatInfo invatianDateFormatInfo = CultureInfo.InvariantCulture.DateTimeFormat;
					int currentIndex = order.BinarySearch(currentDateFormatInfo.FirstDayOfWeek);
					_usualAvailability = new UsualAvailabilityViewModel();
					for (int i = 0; i < 7; i++)
					{
						string dayName = currentDateFormatInfo.GetDayName(order[currentIndex]);
						float availability =
							(from item in usualAvailability
							 where item.DayOfWeek == invatianDateFormatInfo.GetDayName(order[currentIndex])
							 select item.Availability).Single();
						AddDailyAvailabilityViewModel(i, dayName, availability, _usualAvailability);
						currentIndex++;
						if (currentIndex >= 7)
						{
							currentIndex = 0;
						}
					}
				}
				return _usualAvailability;
			}
		}

		/// <summary>
		/// Gets the specific avaialbilities.
		/// </summary>
		/// <value>The specific avaialbilities.</value>
		public SpecificAvailabilityListViewModel SpecificAvailabilities
		{
			get
			{
				if (_specificAvailabilities == null)
				{
					_specificAvailabilities = new SpecificAvailabilityListViewModel(_availabilityManager.GetSpecificAvailabilities());
				}
				return _specificAvailabilities;
			}
		}

		/// <summary>
		/// Gets the new specific availability.
		/// </summary>
		/// <value>The new specific availability.</value>
		public SpecificAvailabilityViewModel NewSpecificAvailability
		{
			get
			{
				if (_newSpecificAvailability == null)
				{
					_newSpecificAvailability = new SpecificAvailabilityViewModel(_availabilityManager.CreateSpecificAvailability(), true);
				}
				return _newSpecificAvailability;
			}
		}

		/// <summary>
		/// Gets the command to save task.
		/// </summary>
		/// <value>The command to save task.</value>
		public ICommand SaveCommand
		{
			get
			{
				if (_saveCommand == null)
				{
					_saveCommand = new Command(parameter => Save());
				}
				return _saveCommand;
			}
		}

		/// <summary>
		/// Gets the add availability command.
		/// </summary>
		/// <value>The add availability command.</value>
		public ICommand AddAvailabilityCommand
		{
			get
			{
				if (_addAvailabilityCommand == null)
				{
					_addAvailabilityCommand = new Command(parameter => AddAvailability(), parameter => CanAddAvailability());
				}
				return _addAvailabilityCommand;
			}
		}
		#endregion

		#region Methods
		/// <summary>
		/// Adds the daily availability view model.
		/// </summary>
		/// <param name="dayNumber">The day number.</param>
		/// <param name="dayName">Name of the day.</param>
		/// <param name="availability">The availability value.</param>
		/// <param name="usualAvailability">The usual availability view model.</param>
		private static void AddDailyAvailabilityViewModel(int dayNumber, string dayName, float availability, UsualAvailabilityViewModel usualAvailability)
		{
			switch (dayNumber)
			{
				case 0:
					usualAvailability.FirstDayOfWeek = new DailyAvailabilityViewModel(dayName, availability);
					break;
				case 1:
					usualAvailability.SecondDayOfWeek = new DailyAvailabilityViewModel(dayName, availability);
					break;
				case 2:
					usualAvailability.ThirdDayOfWeek = new DailyAvailabilityViewModel(dayName, availability);
					break;
				case 3:
					usualAvailability.FourthDayOfWeek = new DailyAvailabilityViewModel(dayName, availability);
					break;
				case 4:
					usualAvailability.FifthDayOfWeek = new DailyAvailabilityViewModel(dayName, availability);
					break;
				case 5:
					usualAvailability.SixthDayOfWeek = new DailyAvailabilityViewModel(dayName, availability);
					break;
				case 6:
					usualAvailability.SeventhDayOfWeek = new DailyAvailabilityViewModel(dayName, availability);
					break;
				default:
					throw new InvalidOperationException();
			}
		}

		/// <summary>
		/// Prepares the availability to save.
		/// </summary>
		/// <param name="usualAvailabilities">The usual availabilities.</param>
		/// <param name="dailyAvailabilityViewModel">The daily availability view model.</param>
		private static void PrepareAvailabilityToSave(Collection<UsualAvailability> usualAvailabilities, DailyAvailabilityViewModel dailyAvailabilityViewModel)
		{
			IEnumerable<DayOfWeek> daysOfWeek =
				from item in Enum.GetNames(typeof(DayOfWeek))
				select (DayOfWeek)Enum.Parse(typeof(DayOfWeek), item);
			DateTimeFormatInfo currentDateFormatInfo = CultureInfo.CurrentUICulture.DateTimeFormat;
			DateTimeFormatInfo invatianDateFormatInfo = CultureInfo.InvariantCulture.DateTimeFormat;
			DayOfWeek currentDayOfWeek =
				(from item in daysOfWeek
				 where currentDateFormatInfo.GetDayName(item).Equals(dailyAvailabilityViewModel.DayName, StringComparison.OrdinalIgnoreCase)
				 select item).Single();
			UsualAvailability currentAvailability =
				(from item in usualAvailabilities
				 where item.DayOfWeek.Equals(invatianDateFormatInfo.GetDayName(currentDayOfWeek), StringComparison.OrdinalIgnoreCase)
				 select item).Single();
			currentAvailability.Availability = dailyAvailabilityViewModel.Availability;
		}

		/// <summary>
		/// Saves this instance.
		/// </summary>
		private void Save()
		{
			Collection<UsualAvailability> usualAvailabilities = _availabilityManager.GetUsualAvailabilities();
			PrepareAvailabilityToSave(usualAvailabilities, _usualAvailability.FirstDayOfWeek);
			PrepareAvailabilityToSave(usualAvailabilities, _usualAvailability.SecondDayOfWeek);
			PrepareAvailabilityToSave(usualAvailabilities, _usualAvailability.ThirdDayOfWeek);
			PrepareAvailabilityToSave(usualAvailabilities, _usualAvailability.FourthDayOfWeek);
			PrepareAvailabilityToSave(usualAvailabilities, _usualAvailability.FifthDayOfWeek);
			PrepareAvailabilityToSave(usualAvailabilities, _usualAvailability.SixthDayOfWeek);
			PrepareAvailabilityToSave(usualAvailabilities, _usualAvailability.SeventhDayOfWeek);
			_availabilityManager.SetUsualAvailabilities(usualAvailabilities);
			ValidateAllTasks();
			OnClosing(EventArgs.Empty);
		}

		/// <summary>
		/// Adds the availability.
		/// </summary>
		private void AddAvailability()
		{
			_availabilityManager.AddSpecificAvailability(_newSpecificAvailability.Model);
			ValidateAllTasks();
			_specificAvailabilities = null;
			OnPropertyChanged("SpecificAvailabilities");
			_newSpecificAvailability = null;
			OnPropertyChanged("NewSpecificAvailability");
		}

		/// <summary>
		/// Validates all tasks.
		/// </summary>
		private void ValidateAllTasks()
		{
			_taskManager.TaskCorrectionsRequested += TaskManager_TaskCorrectionsRequested;
			_taskManager.ValidateAllTasks();
			_taskManager.TaskCorrectionsRequested -= TaskManager_TaskCorrectionsRequested;
		}

		/// <summary>
		/// Handles the TaskCorrectionsRequested event of the TaskManager control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="TaskPlanner.BusinessLogic.Interfaces.TaskCorrectionRequestedEventArgs"/> instance containing the event data.</param>
		private void TaskManager_TaskCorrectionsRequested(object sender, TaskCorrectionRequestedEventArgs e)
		{
			EditTask view = new EditTask();
			TaskDetailsViewModel viewModel = new TaskDetailsViewModel(e.CorrectedTask, _taskManager, _categoryManager);
			viewModel.Closing += (closingSender, closingEventData) => view.Close();
			view.DataContext = viewModel;
			view.ShowDialog();
		}

		/// <summary>
		/// Determines whether this instance can add availability.
		/// </summary>
		/// <returns>
		/// 	<c>true</c> if this instance can add availability; otherwise, <c>false</c>.
		/// </returns>
		private bool CanAddAvailability()
		{
			return _newSpecificAvailability.IsAvailabilitySet && _newSpecificAvailability.IsDateSet;
		}
		#endregion
	}
}
