﻿namespace TaskPlanner.ViewModels
{
	using System.Collections.ObjectModel;
	using System.Windows;
	using System.Windows.Input;
	using TaskPlanner.BusinessLogic.Interfaces;
	using TaskPlanner.Core;
	using TaskPlanner.Entities;
	using TaskPlanner.Properties;

	/// <summary>
	/// Represents the model of the view of the main window.
	/// </summary>
	public class MainWindowViewModel : ViewModelBase
	{
		#region Fields
		private ICommand _createNewTaskCommand;
		private ICommand _viewWorkloadCommand;
		private ITaskManager _taskManager;
		private Collection<Task> _tasksTree;
		private ICommand _deleteTaskCommand;
		private ICommand _editTaskCommand;
		#endregion

		#region Constructors
		/// <summary>
		/// Initializes a new instance of the <see cref="MainWindowViewModel"/> class.
		/// </summary>
		/// <param name="taskManager">The task manager.</param>
		public MainWindowViewModel(ITaskManager taskManager)
		{
			_taskManager = taskManager;
			_createNewTaskCommand = new Command(parameter => CreateNewTask());
			_editTaskCommand = new Command(parameter => ShowEditTaskDialog(parameter as Task));
			_viewWorkloadCommand = new Command(parameter => ViewWorkload());
			_deleteTaskCommand = new Command(parameter => DeleteTask(parameter as Task));
		}
		#endregion

		#region Properties
		/// <summary>
		/// Gets the command to create new task.
		/// </summary>
		/// <value>The command to create new task.</value>
		public ICommand CreateNewTaskCommand
		{
			get
			{
				return _createNewTaskCommand;
			}
		}

		/// <summary>
		/// Gets the edit task command.
		/// </summary>
		/// <value>The command to edit task.</value>
		public ICommand EditTaskCommand
		{
			get
			{
				return _editTaskCommand;
			}
		}

		/// <summary>
		/// Gets the view workload command.
		/// </summary>
		/// <value>The command to view workload.</value>
		public ICommand ViewWorkloadCommand
		{
			get
			{
				return _viewWorkloadCommand;
			}
		}

		/// <summary>
		/// Gets the delete task command.
		/// </summary>
		/// <value>The command to delete task.</value>
		public ICommand DeleteTaskCommand
		{
			get
			{
				return _deleteTaskCommand;
			}
		}

		/// <summary>
		/// Gets the tasks tree.
		/// </summary>
		/// <value>The tasks tree.</value>
		public Collection<Task> TasksTree
		{
			get
			{
				if (_tasksTree == null)
				{
					_tasksTree = _taskManager.GetTasksTree();
				}
				return _tasksTree;
			}
		}
		#endregion

		#region Methods
		/// <summary>
		/// Views the workload.
		/// </summary>
		private void ViewWorkload()
		{
			ViewWorkload view = new ViewWorkload();
			IAvailabilityManager availabilityManager = ServiceLocator.Instance.Resolve<IAvailabilityManager>();
			ITaskManager taskManager = ServiceLocator.Instance.Resolve<ITaskManager>();
			ICategoryManager categoryManager = ServiceLocator.Instance.Resolve<ICategoryManager>();
			WorkloadViewModel viewModel = new WorkloadViewModel(availabilityManager, taskManager, categoryManager);
			viewModel.Closing += (sender, e) => view.Close();
			view.DataContext = viewModel;
			view.ShowDialog();
			_tasksTree = null;
			OnPropertyChanged("TasksTree");
		}

		/// <summary>
		/// Creates the new task.
		/// </summary>
		private void CreateNewTask()
		{
			ShowEditTaskDialog(_taskManager.CreateNewTask());
		}

		/// <summary>
		/// Deletes the task.
		/// </summary>
		/// <param name="task">The task to be deleted.</param>
		private void DeleteTask(Task task)
		{
			if (MessageBox.Show(Resources.DeleteTaskConfirmation, Resources.ConfirmationCaption, MessageBoxButton.YesNo) == MessageBoxResult.Yes)
			{
				_taskManager.DeleteTask(task);
				_tasksTree = null;
				OnPropertyChanged("TasksTree");
			}
		}

		/// <summary>
		/// Shows the edit task dialog.
		/// </summary>
		/// <param name="task">The task to be edited.</param>
		private void ShowEditTaskDialog(Task task)
		{
			EditTask view = new EditTask();
			ICategoryManager categoryManager = ServiceLocator.Instance.Resolve<ICategoryManager>();
			TaskDetailsViewModel viewModel = new TaskDetailsViewModel(task, _taskManager, categoryManager);
			viewModel.Closing += (sender, e) => view.Close();
			view.DataContext = viewModel;
			view.ShowDialog();
			_tasksTree = null;
			OnPropertyChanged("TasksTree");
		}
		#endregion
	}
}
