﻿namespace TaskPlanner.ViewModels
{
	using System;
	using System.Collections.ObjectModel;
	using System.Windows.Input;
	using TaskPlanner.BusinessLogic.Interfaces;
	using TaskPlanner.Entities;

	/// <summary>
	/// Represents the model of the view for task details.
	/// </summary>
	public class TaskDetailsViewModel : ViewModelBase
	{
		#region Fields
		private Task _task;
		private bool _isTaskDueDateSet;
		private ICommand _saveCommand;
		private ITaskManager _taskManager;
		private ICategoryManager _categoryManager;
		#endregion

		#region Constructors
		/// <summary>
		/// Initializes a new instance of the <see cref="TaskDetailsViewModel"/> class.
		/// </summary>
		/// <param name="task">The task to be represented by this view model.</param>
		/// <param name="taskManager">The task manager.</param>
		/// <param name="categoryManager">The category manager.</param>
		public TaskDetailsViewModel(Task task, ITaskManager taskManager, ICategoryManager categoryManager)
		{
			_task = task;
			_taskManager = taskManager;
			_categoryManager = categoryManager;
		}
		#endregion

		#region Properties
		/// <summary>
		/// Gets the available parent tasks.
		/// </summary>
		/// <value>The available parent tasks.</value>
		public ObservableCollection<Task> AvailableParentTasks
		{
			get
			{
				return new ObservableCollection<Task>(_taskManager.GetAllTasks());
			}
		}

		/// <summary>
		/// Gets the available categories.
		/// </summary>
		/// <value>The available categories.</value>
		public ObservableCollection<string> AvailableCategories
		{
			get
			{
				return new ObservableCollection<string>(_categoryManager.GetAllCategories());
			}
		}

		/// <summary>
		/// Gets or sets the name of the task.
		/// </summary>
		/// <value>The name of the task.</value>
		public string TaskName
		{
			get
			{
				return _task.TaskName;
			}

			set
			{
				_task.TaskName = value;
			}
		}

		/// <summary>
		/// Gets or sets the completed percentage.
		/// </summary>
		/// <value>The completed percentage.</value>
		public int CompletedPercentage
		{
			get
			{
				return _task.CompletedPercentage;
			}

			set
			{
				_task.CompletedPercentage = value;
			}
		}

		/// <summary>
		/// Gets or sets the estimated time.
		/// </summary>
		/// <value>The estimated time.</value>
		public float EstimatedTime
		{
			get
			{
				return _task.EstimatedTime;
			}

			set
			{
				_task.EstimatedTime = value;
				TaskCorrectionResult taskCorrection = _taskManager.ApplyEstimatedTimeChange(_task, _isTaskDueDateSet);
				if (taskCorrection.IsCorrectionMade)
				{
					_task.DueDate = taskCorrection.CorrectedTask.DueDate;
					OnPropertyChanged("DueDate");
				}
			}
		}

		/// <summary>
		/// Gets or sets the completed time.
		/// </summary>
		/// <value>The completed time.</value>
		public float CompletedTime
		{
			get
			{
				return _task.CompletedTime;
			}

			set
			{
				_task.CompletedTime = value;
			}
		}

		/// <summary>
		/// Gets or sets the start date.
		/// </summary>
		/// <value>The start date.</value>
		public DateTime StartDate
		{
			get
			{
				return _task.StartDate;
			}

			set
			{
				_task.StartDate = value;
			}
		}

		/// <summary>
		/// Gets or sets the due date.
		/// </summary>
		/// <value>The due date.</value>
		public DateTime DueDate
		{
			get
			{
				return _task.DueDate;
			}

			set
			{
				_isTaskDueDateSet = true;
				_task.DueDate = value;
			}
		}

		/// <summary>
		/// Gets or sets the end date.
		/// </summary>
		/// <value>The end date.</value>
		public DateTime EndDate
		{
			get
			{
				return _task.EndDate;
			}

			set
			{
				_task.EndDate = value;
			}
		}

		/// <summary>
		/// Gets or sets the category.
		/// </summary>
		/// <value>The category.</value>
		public string Category
		{
			get
			{
				return _task.Category;
			}

			set
			{
				_task.Category = value;
			}
		}

		/// <summary>
		/// Gets or sets the parent task.
		/// </summary>
		/// <value>The parent task.</value>
		public Task ParentTask
		{
			get
			{
				return _task.Parent;
			}

			set
			{
				_task.Parent = value;
			}
		}

		/// <summary>
		/// Gets the command to save task.
		/// </summary>
		/// <value>The command to save task.</value>
		public ICommand SaveCommand
		{
			get
			{
				if (_saveCommand == null)
				{
					_saveCommand = new Command(parameter => Save());
				}
				return _saveCommand;
			}
		}
		#endregion

		#region Methods
		/// <summary>
		/// Saves this instance.
		/// </summary>
		private void Save()
		{
			_taskManager.SaveTask(_task);
		}
		#endregion
	}
}
