﻿namespace TaskPlanner.ViewModels
{
	using System;
	using System.ComponentModel;
	using System.Diagnostics;
	using System.Windows.Input;

	/// <summary>
	/// Represents the base class for models of views.
	/// </summary>
	public abstract class ViewModelBase : INotifyPropertyChanged
	{
		#region Fields
		private ICommand _closeCommand;
		#endregion

		#region Events
		/// <summary>
		/// Occurs when a property value changes.
		/// </summary>
		public event PropertyChangedEventHandler PropertyChanged;

		/// <summary>
		/// Occurs when model is closing.
		/// </summary>
		public event EventHandler Closing;
		#endregion

		#region Properties
		/// <summary>
		/// Gets the close command.
		/// </summary>
		/// <value>The close command.</value>
		public ICommand CloseCommand
		{
			get
			{
				if (_closeCommand == null)
				{
					_closeCommand = new Command(parameter => OnClosing(EventArgs.Empty));
				}
				return _closeCommand;
			}
		}
		#endregion

		#region Methods
		/// <summary>
		/// Called when property is changed.
		/// </summary>
		/// <param name="propertyName">Name of the property.</param>
		protected virtual void OnPropertyChanged(string propertyName)
		{
			VerifyPropertyName(propertyName);
			PropertyChangedEventHandler handler = PropertyChanged;
			if (handler != null)
			{
				PropertyChangedEventArgs e = new PropertyChangedEventArgs(propertyName);
				handler(this, e);
			}
		}

		/// <summary>
		/// Raises the <see cref="E:Closing"/> event.
		/// </summary>
		/// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
		protected virtual void OnClosing(EventArgs e)
		{
			EventHandler hander = Closing;
			if (hander != null)
			{
				hander(this, e);
			}
		}

		/// <summary>
		/// Verifies the name of the property.
		/// </summary>
		/// <param name="propertyName">Name of the property.</param>
		[Conditional("DEBUG")]
		private void VerifyPropertyName(string propertyName)
		{
			if (TypeDescriptor.GetProperties(this)[propertyName] == null)
			{
				string message = string.Concat("Invalid property name: ", propertyName);
				throw new InvalidOperationException(message);
			}
		}
		#endregion
	}
}
